<?php
/*
Plugin Name: Companion Portfolio
Plugin URI:  http://codeermeneer.nl/portfolio/companion-portfolio/
Description: A fully responsive portfolio plugin, looking sharp on every device! Easy to use and fully configurable.
Version:     2.4.0.1
Author:      Papin Schipper
Author URI:  http://codeermeneer.nl
License:     GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: companion-portfolio
Domain Path: /languages/

The WordPress plugin Companion Sitemap Generator is licensed under the GPL v2 or later.
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License, version 2, as published by the Free Software Foundation.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

// Disable direct access
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

// (De)Activation settings
register_activation_hook( __FILE__, 'flush_rewrite_rules' );
register_deactivation_hook( __FILE__, 'flush_rewrite_rules' );

// Database version
function cp_version() {
	return '2.3.0';
}

// On plugin update
function cp_update_db_check() {
    if ( get_site_option( 'cp_version' ) != cp_version() ) {
        update_option( "cp_version", cp_version() );
    }
}
add_action( 'upgrader_process_complete', 'cp_update_db_check' );

// Init function
function cp_portfolio_init() {

	// Load translations
	load_plugin_textdomain( 'companion-portfolio', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' ); 

	// Add custom image size
	add_image_size( 'portfolio-image', 400, 300, true  );

	// Create custom taxs
	cp_custom_taxs();

	// Create custom post type
	cp_custom_pt();

	// Glue them together
	cp_taxs_pt_glue();


}
add_action( 'init', 'cp_portfolio_init', 1 );

// Register custom post type
function cp_custom_pt() {

	// Set up variables
	$slu = get_option( 'cp_cpt_base' ) ? get_option( 'cp_cpt_base' ) : 'portfolio';

    // Register post type
    register_post_type( 'portfolio',
        array(
	        'labels' => array(
				'name' 					=> __( 'Portfolio', 'companion-portfolio' ),
				'singular_name' 		=> __( 'Portfolio', 'companion-portfolio' ),
				'add_new' 				=> __( 'New item', 'companion-portfolio'),
				'add_new_item' 			=> __( 'Add new item', 'companion-portfolio' ),
				'new_item' 				=> __( 'New item', 'companion-portfolio' ),
				'all_items' 			=> __( 'All items', 'companion-portfolio' ),
				'search_items' 			=> __( 'Search portfolio', 'companion-portfolio' ),
				'not_found' 			=> __( 'No items found', 'companion-portfolio'),
				'not_found_in_trash' 	=> __( 'No items found in Trash', 'companion-portfolio' ),
				'edit_item'	 			=> __( 'Edit item', 'companion-portfolio' ),
				'view_item'				=> __( 'View item', 'companion-portfolio' ),
	        ),
	        'public' 		=> true,
	        'has_archive' 	=> true,
	        'menu_position' => 25, // Below comments
	        'rewrite' 		=> array( 'slug' => $slug ),
	        'supports' 		=> array( 
	        	'title', 
	        	'editor', 
	        	'thumbnail', 
	        	'excerpt', 
	        	'comments',
	        ),
	        'show_in_rest' 	=> true, // Enable gutenberg
	        'menu_icon' 	=> 'dashicons-category',
        )
    ); 

}

// Create taxonomy portfolio categories
function cp_custom_taxs() {

	// Create category taxonomy
    register_taxonomy( 'portfolio_cat', 'portfolio', array(
		"hierarchical" 		=> true,
		"labes"				=> array(
	        'name'              => __( 'Categories' ),
	        'singular_name'     => __( 'Category' ),
	        'search_items'      => __( 'Search categories' ),
	        'all_items'         => __( 'All Categories' ),
	        'edit_item'         => __( 'Edit Categories' ),
	        'update_item'       => __( 'Update Categories' ),
	        'add_new_item'      => __( 'Add New Categories' ),
	        'new_item_name'     => __( 'New Categories Name' ),
	        'menu_name'         => __( 'Categories' ),
	    ),
		'query_var' 		=> true,
		'rewrite' 			=> array( 
			'slug' 			=> 'portfolio-category', 
			'with_front' 	=> true 
		),
		'public' 			=> true,
		'show_ui' 			=> true,
		'show_tagcloud' 	=> false,
		'_builtin' 			=> false,
		'show_in_nav_menus' => false
    ));

	// Create tags taxonomy
    register_taxonomy( 'portfolio_tag', 'portfolio', array(
		"hierarchical" 		=> false,
		"labes"				=> array(
	        'name'              => __( 'Tags' ),
	        'singular_name'     => __( 'Tags' ),
	        'search_items'      => __( 'Search tags' ),
	        'all_items'         => __( 'All Tags' ),
	        'edit_item'         => __( 'Edit Tags' ),
	        'update_item'       => __( 'Update Tags' ),
	        'add_new_item'      => __( 'Add New Tags' ),
	        'new_item_name'     => __( 'New Tags Name' ),
	        'menu_name'         => __( 'Tags' ),
	    ),
		'query_var' 		=> true,
		'rewrite' 			=> array( 
			'slug' 			=> 'portfolio-tag', 
			'with_front' 	=> true 
		),
		'public' 			=> true,
		'show_ui' 			=> true,
    ));

    // Create default category to prevent errors
    cp_default_category();

}

// Glue them together
function cp_taxs_pt_glue() {
    register_taxonomy_for_object_type( 'portfolio_category', 'portfolio' );
    register_taxonomy_for_object_type( 'portfolio_tag', 'portfolio' );
}

// Create default category to prevent errors
function cp_default_category() {
    if( !term_exists( __( 'Default', 'companion-portfolio' ), 'portfolio_cat' ) ) {
	    wp_insert_term(
	    	__( 'Default', 'companion-portfolio' ),
	    	'portfolio_cat',
	    	array(
	    		'description'	=> __( "The default portfolio category, will be used if no category is selected. It's advised to not remove this category.", "companion-portfolio" ),
	    		'slug'			=> 'companion-portfolio'
	    	)
	    );
	}
}

// Configure permalinks
function cp_reg_permalink( $permalink, $post, $leavename, $sample ) {

    if ( $post->post_type == 'portfolio' && get_option( 'permalink_structure' ) ) {

    	// Base slug
    	$slug 			= get_option( 'cp_cpt_base' ) ? get_option( 'cp_cpt_base' ) : 'portfolio';
        $struct 		= '/'.$slug.'/%postname%/';

        // Category
        $portfolio_cat 	= get_the_terms( $post->post_id, 'portfolio_cat' );
        $rewrite_cat 	= !empty( $portfolio_cat ) ? esc_html( $portfolio_cat[0]->slug ) : 'companion-portfolio';

        // Custom options
        $rewritecodes 	= array( '%category%', '%postname%' );
        $replacements 	= array( $rewrite_cat, $post->post_name );

        // Finish the permalink
        $permalink = home_url( str_replace( $rewritecodes, $replacements, $struct ) );
        $permalink = user_trailingslashit( $permalink, 'single' );

    }

    return $permalink;

}
add_filter( 'post_type_link', 'cp_reg_permalink', 10, 4 );

// Add the URL Metabox
function cp_porfolio_url_metaboxes() {
	add_meta_box( 'cp_portfolio-category', __( 'Categories' ), 'cp_portfolio_category', 'portfolio', 'side', 'default' );
	add_meta_box( 'cp_portfolio-tag', __( 'Tags' ), 'cp_portfolio_tag', 'portfolio', 'side', 'default' );
}
add_action( 'add_meta_boxes', 'cp_porfolio_url_metaboxes' );

// The category metabox
function cp_portfolio_category() {

	global $post;

	$all_port_cats 		= get_terms( 'portfolio_cat', array( 'hide_empty' => false ) );
    $cur_port_cats 		= get_the_terms( $post->ID, 'portfolio_cat' );
    $cur_port_cats_ids 	= wp_list_pluck( $cur_port_cats, 'term_id' );

	foreach ( $all_port_cats as $port_cat ) {
		$checked = in_array( $port_cat->term_id, $cur_port_cats_ids ) ? 'CHECKED' : '';
		echo "<p><label><input type='checkbox' value='$port_cat->term_id' name='portfolio_categories[]' $checked >$port_cat->name</label></p>";
	}

}

// The category metabox
function cp_portfolio_tag() {

	global $post;

	$all_port_tags 		= get_terms( 'portfolio_tag', array( 'hide_empty' => false ) );
    $cur_port_tags 		= get_the_terms( $post->ID, 'portfolio_tag' );
    $cur_port_tags_ids 	= wp_list_pluck( $cur_port_tags, 'term_id' );

	foreach ( $all_port_tags as $port_tags ) {
		$checked = in_array( $port_tags->term_id, $cur_port_tags_ids ) ? 'CHECKED' : '';
		echo "<p><label><input type='checkbox' value='$port_tags->name' name='portfolio_tags[]' $checked >$port_tags->name</label></p>";
	}

}

// Save the Metabox Data
function cp_save_custom_meta( $post_id, $post ) {

	if ( $post->post_type == 'portfolio' ) {

		// Categories
		if( isset( $_POST['portfolio_categories'] ) ) {
			foreach ( $_POST['portfolio_categories'] as $selected_categorie ) $portfolio_cats[] = $selected_categorie;

		// Else we force the default one
		} else {
			$portfolio_cats[] = get_term_by( 'name', __( 'Default', 'companion-portfolio' ), 'portfolio_cat' );

		}

		wp_set_post_terms( $post_id, $portfolio_cats, 'portfolio_cat' );

		// Tags
		if( isset( $_POST['portfolio_tags'] ) ) {
			foreach ( $_POST['portfolio_tags'] as $selected_tag ) $portfolio_tags[] = $selected_tag;
		} else {
			$portfolio_tags[] = '';
		}

		wp_set_post_terms( $post_id, $portfolio_tags, 'portfolio_tag' );

	}
	

}
add_action( 'save_post', 'cp_save_custom_meta', 1, 2 );

// The shortcode
function cp_frontEnd_loop_handler( $attributes ) {

	// Allowed attributes
	$conf = shortcode_atts([
		'limit' 		=> '-1',
		'sortby' 		=> 'name',
		'columns' 		=> '2',
		'showdate' 		=> 'true',
		'showexcerpt' 	=> 'false',
		'showcategory'	=> 'false',
		'order' 		=> 'ASC',
		'cat' 			=> '',
		'styling' 		=> 'blank',
	], $attributes, 'companion-portfolio' );

	return cp_frontEnd_loop( $conf['limit'], $conf['sortby'], $conf['columns'], $conf['showdate'], $conf['showexcerpt'], $conf['showcategory'], $conf['order'], $conf['cat'], $conf['styling'] );
}
function cp_frontEnd_loop( $limit, $sortby, $columns, $showdate, $showexcerpt, $showcategory, $order, $cat, $styling ) {
	
	// Set up variables, sanitize them
	$columns 			= sanitize_text_field( $columns );
	$sortby 			= sanitize_text_field( $sortby );
	$limit 				= sanitize_text_field( $limit );
	$order 				= sanitize_text_field( $order );
	$category 			= sanitize_text_field( $cat );
	$tax_query 			= ( $cat != '' ) ? array( array( 'taxonomy' => 'portfolio_cat', 'field' => 'name', 'terms' => $category ) ) : '';
	$cat_class 			= ( $cat != '' ) ? 'cat-' . str_replace( "-", " ", $category ) : '';
	$styling 			= sanitize_text_field( $styling );

	// Set up arguments
	$args = array(
		'post_type' 		=> 'portfolio', 
		'post_status' 		=> 'publish', 
		'orderby' 			=> $sortby, 
		'posts_per_page' 	=> $limit, 
		'order' 			=> $order,
		'tax_query' 		=> $tax_query
	);

	// Return the items
	$portfolio_query 	= new WP_Query( $args );

	// Open the portfolio container
	$return = "<div id='cp_portfolio_container' class='cp_portfolio_container cp_portfolio-columns-{$columns} style-{$styling}'>";

	if( $portfolio_query->have_posts() ) {
		while( $portfolio_query->have_posts() ) {

			$portfolio_query->the_post(); 

			// Basic vars
			$imageurl 		= wp_get_attachment_image( get_post_thumbnail_id( get_the_ID() ), 'portfolio-image' );
			$portfolio_cat 	= get_the_terms( get_the_ID(), 'portfolio_cat' );
			$permalink 		= get_the_permalink();
			$category 		= !empty( $portfolio_cat ) ? esc_html( $portfolio_cat[0]->name ) : '';

			$return .= '<a class="cp_portfolio_item cp_columns-'.$columns.' '.$cat_class.'" href="'.$permalink.'">';

				ob_start();
					do_action( 'cp_portfolio_before_image' );
			        $cp_portfolio_before_image = ob_get_contents();
			    ob_end_clean();
			    $return .= $cp_portfolio_before_image;

				// Thumbnail
				$return .= '<div class="cp_portfolio_image">'.$imageurl.'</div>';

				ob_start();
					do_action( 'cp_portfolio_after_image' );
					do_action( 'cp_portfolio_before_text' );
			        $cp_portfolio_before_text = ob_get_contents();
			    ob_end_clean();

			    $return .= $cp_portfolio_before_text;

				// Text
				$return .= '<div class="cp_portfolio_text">';

						$return .= '<h2 class="cp_portfolio_title">'.get_the_title().'</h2>';

						if( $showcategory == 'true' && $category != '' ) $return .= '<h3 class="cp_portfolio_category">'.$category.'</h3>';
						if( $showexcerpt == 'true') 	$return .= '<p class="cp_portfolio_excerpt">'.get_the_excerpt().'</p>';
						if( $showdate == 'true' ) 		$return .= '<p class="cp_portfolio_date"><strong>'.get_the_date( 'j' ).'</strong> '.get_the_date( 'F Y' ).'</p>';

				$return .= '</div>';

				ob_start();
					do_action( 'cp_portfolio_after_text' );
			        $cp_portfolio_after_text = ob_get_contents();
			    ob_end_clean();
			    $return .= $cp_portfolio_after_text;

			$return .= '</a>';

		}

	} else {

		$return .= '<div class="cp_portfolio_message no-projects-found"><p>'.__( 'Oops: No portfolio items were found', 'companion-portfolio' ).'.</p></div>';

	}

	$return .= '</div>';

	wp_reset_postdata();

	return $return;

}
add_shortcode( 'companion-portfolio' , 'cp_frontEnd_loop_handler' );

// Adds styling
function cp_frontend_style() {
	wp_enqueue_style( 'cp-styling', plugin_dir_url( __FILE__ ).'frontend/style.css', false, '1.0.0', 'all'  );
}
add_action( 'wp_enqueue_scripts', 'cp_frontend_style' );;

// Add table items
function add_cp_columns( $plugin_columns ) {

    $new_columns['cb'] 				= '<input type="checkbox" />';
    $new_columns['title'] 			= __( 'Title' );
    $new_columns['author'] 			= __( 'Author' );
    $new_columns['portfolio_cat'] 	= __( 'Categories' );
    $new_columns['portfolio_tag'] 	= __( 'Tags' );
    $new_columns['date'] 			= __( 'Date' );
 
    return $new_columns;
}
add_filter( 'manage_edit-portfolio_columns', 'add_cp_columns' );

// Create columns for custom taxonym portfolio_cat
function cp_set_custom_columns( $columns ) {
    $new_columns['portfolio_cat'] 	= __( 'Categories' );
    $new_columns['portfolio_tag'] 	= __( 'Tags' );
    return $columns;
}
function cp_custom_column( $column, $post_id ) {
    switch ( $column ) {
        case 'portfolio_cat' :
            $terms = get_the_term_list( $post_id , 'portfolio_cat' , '' , ',' , '' );
            echo is_string( $terms ) ? $terms : "&#8212;";
            break;
        case 'portfolio_tag' :
            $terms = get_the_term_list( $post_id , 'portfolio_tag' , '' , ',' , '' );
            echo is_string( $terms ) ? $terms : "&#8212;";
            break;
    }
}
add_filter( 'manage_portfolio_posts_columns', 'cp_set_custom_columns' );
add_action( 'manage_portfolio_posts_custom_column' , 'cp_custom_column', 10, 2 );

// Add permalink settings to default permalink page
function cp_load_permalinks() {

	// Portfolio archive
    // if( isset( $_POST['cp_cpt_archive'] ) ) update_option( 'cp_cpt_archive', esc_attr( $_POST['cp_cpt_archive'] ) );
    // add_settings_field( 'cp_cpt_archive', __( 'Portfolio archives', 'companion-portfolio' ), 'cp_field_callback', 'permalink', 'optional' );

	// Single portfolio URL
    if( isset( $_POST['cp_cpt_base'] ) ) update_option( 'cp_cpt_base', esc_attr( $_POST['cp_cpt_base'] ) );
    add_settings_field( 'cp_cpt_base', __( 'Portfolio pages', 'companion-portfolio' ), 'cp_field_callback', 'permalink', 'optional' );

}
add_action( 'load-options-permalink.php', 'cp_load_permalinks' );

// The input field
function cp_field_callback() {

	// Portfolio archive
    // $cp_cpt_archive = get_option( 'cp_cpt_archive' );  
    // echo '<code>'.get_site_url().'/</code><input type="text" value="' . esc_attr( $cp_cpt_archive ) . '" name="cp_cpt_archive" id="cp_cpt_archive" class="regular-text" placeholder="portfolio"/><code>/project-name/</code>';

	// Single portfolio URL
    $cp_cpt_base = get_option( 'cp_cpt_base' );  
    echo '<code>'.get_site_url().'/</code><input type="text" value="' . esc_attr( $cp_cpt_base ) . '" name="cp_cpt_base" id="cp_cpt_base" class="regular-text" placeholder="portfolio"/><code>/project-name/</code>';
    echo '<p>'.__( 'You can also use:', 'companion-portfolio' ).' <code>%category%</code></p>';

}
add_action( 'init', 'cp_custom_pt' );

// Skip block registration if Gutenberg is not enabled/merged.
if ( function_exists( 'register_block_type' ) ) {
	require( 'cp_gutenberg.php' );
}

